from os import PathLike
from abc import ABC, abstractmethod
from typing import List, Optional, Union

# copied from: https://github.com/python/cpython/blob/3.8/Lib/_collections_abc.py
def _check_methods(C, *methods):
    mro = C.__mro__
    for method in methods:
        for B in mro:
            if method in B.__dict__:
                if B.__dict__[method] is None:
                    return NotImplemented
                break
        else:
            return NotImplemented
    return True


class Downloader(ABC):
    """ ABC for file downloaders. Implementing classes shall download files
        from a given storage location to be preprocessed.
    """

    @abstractmethod
    def download(self, remote_path: PathLike, local_path: PathLike) -> PathLike:
        pass


class Uploader(ABC):
    @abstractmethod
    def upload(self, local_path: Union[PathLike, List[PathLike]], remote_dir: PathLike) -> List[PathLike]:
        pass

    @abstractmethod
    def product_exists(self, remote_dir: PathLike) -> bool:
        pass
